unit _3DR;

(* 3DR interface unit *)
(* Works with Delphi 1.0, 2.0 *)

interface

(*
// Dictionary of Acronyms, Keywords etc.,
//
//      R3d     - 3DR, misspelt, for compilation reasons.
//
//      RC      - Rendering Context, as in RCInfo
//      Pt      - Point, as in PtLong_T
//      Clr     - Color, as in ClrLong_T
//      Ptr     - Pointer, as in LongPtrToByte_T
//      Prm     - Primitive, as in R3dPrmType_T
//      Vtx     - Vertex, as in R3dVtxType_T
//      SRC     - Source, as in R3D_STATE_SRC_BLEND
//      DST     - Destination, as in R3D_STATE_DST_BLEND
//      ROP     - Raster (logical) Operation, as in R3D_STATE_ROP_FUNCTION
//      Msg     - Message, as in R3dMsgType
//      MSG     - Message, as in R3D_MSG_NOP
//      Pkt     - Packet, as in R3dPktType
//      PKT     - Packet, as in R3D_PKT_END
//      Hdr     - Header, as in MsgHdrType
//      Gen     - Generic, as in MsgGenHdr
//      Bmp     - Bitmap, as in MsgBmpHdr
//      Sta     - State, as in PktStaType
//      STA     - STATE, as in R3D_STA_ENABLES
//      DB      - DoubleBuffer, as in  R3D_DB_FRONT
//      ZF      - Z buffer Function, as in R3D_ZF_NEVER 
//      LT      - Less Then, as in R3D_ZF_LT
//      EQ      - EQual to, as in R3D_ZF_EQ
//      LE      - Less then or Equal to, as in R3D_ZF_LE
//      GT      - Greater Then, as in R3D_ZF_GT
//      NE      - Not Equal to, as in R3D_ZF_NE
//      GE      - Greater then or Equal to, as in R3D_ZF_GE
//      BL      - Blending Function, as in R3D_BL_SRC_INV 
//      INV     - INVerse, as in R3D_BL_SRC_INV
//      RP      - Raster oPeration function, as in R3D_RP_AND 
//      TF      - Texture Filter function, as in R3D_TF_MIPMAP 
//      TM      - Texture Modulation function, as in R3D_TF_MIPMAP
//      TT      - Texture Type information , as in R3D_TT_KEEP
//      BT      - Bitmap Type information , as in R3D_BT_ALPHA
//      ST      - State Type information , as in R3D_ST_LEFT
//      CT      - Clearbuffer Type information , as in R3D_CT_DRAWBUFFER 
*)

(*
// Typedefs used commonly by 3dr.
*)
type
  TFixed16 = word;
  TFixed32 = longint;

  TR3dHandle = longint;

  PByte = ^byte;

(*D*
// 3DR data types - Points
*D*)

type
  TR3dClr = object
    r,g,b,a : TFixed16;
  end;

  TR3dFixedVtx = object
    x,y,z : TFixed32;
  end;

  TR3dFixedVtxClr = object(TR3dFixedVtx)
    r,g,b,a : TFixed16;
  end;

  TR3dFixedVtxTex = object(TR3dFixedVtx)
    q,u,v : TFixed32;
  end;

  TR3dFixedVtxTexClr = object(TR3dFixedVtxTex)
    r,g,b,a : TFixed16;
  end;

  TR3dFloatVtx = object
    x,y,z : single;
  end;

  TR3dFloatVtxClr = object(TR3dFloatVtx)
    r,g,b,a : TFixed16;
  end;

  TR3dFloatVtxTex = object(TR3dFloatVtx)
    q,u,v : TFixed32;
  end;

  TR3dFloatVtxTexClr = object(TR3dFloatVtxTex)
    r,g,b,a : TFixed16;
  end;

(*D*
// 3DR data types - Message Header
*D*)

type
  TR3dMsgHdr = object
    MsgHdrType : word;
    MsgHdrVersion : word;
    MsgHdrSize : longint;
    MsgHdrRC : TR3dHandle;
  end;

(*D*
// 3DR data types - Generic Message definition
*D*)

  TR3dMsgGen = object(TR3dMsgHdr)
    MsgGenInfo0 : word;
    MsgGenInfo1 : word;
    MsgGenData : longint;
    MsgGenBuffer : array[0..0] of longint;
  end;

(*D*
// 3DR data types - Texture Message definition
*D*)

  TR3dMsgTex = object(TR3dMsgHdr)
    MsgTexInfo : word;
    MsgTexBitsPixel : word;
    MsgTexMipLevel : word;
    MsgTexWidth : word;
    MsgTexRows : word;
    MsgTexWidthBytes : smallint;
    MsgTexId : longint;
    MsgTexData : longint;                     (* Buffer-wide Modifier *)
    MsgTexBuffer : array[0..0] of longint;        (* variable size data   *)
  end;

(*D*
// 3DR data types - Bitmap Message definition
*D*)

  TR3dMsgBMP = object(TR3dMsgHdr)
    MsgBmpPad : word;
    MsgBmpInfo : word;
    MsgBmpBitsPixel : word;
    MsgBmpWidth : word;
    MsgBmpHeight : word;
    MsgBmpWidthBytes,
    MsgBmpSrcX,
    MsgBmpSrcY,
    MsgBmpDstX,
    MsgBmpDstY : smallint;
    MsgBmpData : longint;                     (* Buffer-wide Modifier *)
    MsgBmpBuffer : array[0..0] of longint;        (* variable size data   *)
  end;

(*D*
// 3DR data types - Packet definition
*D*)

  TR3dMsgPkt = object(TR3dMsgHdr)
    MsgPktBuffer : array[0..0] of longint;        (* variable size data   *)
  end;

(*D*
// 3DR data types - Packet (of primitives) definition
*D*)

  TR3dPktPrm = object
    PktPrmPktType,
    PktPrmVtxInfo,
    PktPrmVtxNum,
    PktPrmVtxSize : word;
    PktPrmBuffer : array[0..0] of longint;    (* variable size data   *)
  end;

(*D*
// 3DR data types - Packet (of State Information) definition
*D*)

  TR3dPktSta = object
    PktStaPktType : word;
    PktStaState : word;
    PktStaData : longint;
  end;

(*D*
// 3DR data types - Packet (of Vertices) definition
*D*)

  TR3dPktGen = object
    PktGenPktType : word;
    PktGenInfo : word;
  end;

{$IFDEF WIN32}

  TR3dBuffer = object
    R3dBufBasePtr : PByte;			(* Base of LFB 			*)
    R3dBufWidth : word;			(* width  of buffer, pix*)
    R3dBufHeight : word;			(* height of buffer, pix*)
    R3dBufBitsPixel : word;		(* 1, 8, 16, 24, 32		*)
    R3dBufWidthBytes : smallint;		(* stride, (signed)		*)

    R3dBufMinY : word;
    R3dBufMaxY : word;
    R3dBufBufInfo : word;
    R3dBufMask : array[0..3] of word;			(* masks for RGBA.		*)
    R3dBufMax : array[0..3] of word;			(* max. each for RGBA	*)

    R3dBufBufAccess : pointer;		(* see comments above.	*)
    R3dBufBufCopySpans : pointer;		(* see comments above.	*)
    R3dBufBufCopyPixels : pointer;	(* see comments above.	*)

    R3dBufRealBase : PByte;			(* true Base of LFB.	*)
    R3dBufDevPrivate : pointer;		(* private to device.	*)
    R3dBufEngPrivate : pointer;		(* private to engine.	*)
  end;

{$ENDIF}

(*
// 3DR data types - far pointer types for above structures.
// FAR versions of the above for 16 bit compilers,
// FAR is reduced to NULL for 32 bit compilers.
*)

(*
// 3DR - Definitions
*)

(*
// These are the various packet identifers used by 3DR.
//      These are used to create general packets.
//      These values are used in R3dMsgPtr_t.
// Pick ONE of ...
*)

const
  R3D_PKT_END                        =     0;
  R3D_PKT_SETSTATE                   =     1;
  R3D_PKT_CLEARBUFFERS               =     2;
  R3D_PKT_SWAPBUFFERS                =     3;
  R3D_PKT_POINTS                     =     4;
  R3D_PKT_LINES                      =     5;
  R3D_PKT_POLYLINE                   =     6;
  R3D_PKT_TRIANGLES                  =     7;
  R3D_PKT_TRISTRIP                   =     8;
  R3D_PKT_POLYGON                    =     9;
  R3D_PKT_QUADS                      =    10;
  R3D_PKT_QUADSTRIP                  =    11;

(*
// These are used by R3dPrimitive()
// Pick ONE of ...
*)
  R3D_PRM_POINTS                     =    R3D_PKT_POINTS;
  R3D_PRM_LINES                      =    R3D_PKT_LINES;
  R3D_PRM_POLYLINE                   =    R3D_PKT_POLYLINE;
  R3D_PRM_TRIANGLES                  =    R3D_PKT_TRIANGLES;
  R3D_PRM_TRISTRIP                   =    R3D_PKT_TRISTRIP;
  R3D_PRM_POLYGON                    =    R3D_PKT_POLYGON;

(*
// These are used by R3dGetState() and R3dSetState()
// Pick ONE of ...
*)

(*
// Enumerated states.
// >> Possible values mentioned in the comments section.
// Pick ONE of ...
*)
  R3D_STA_ENABLES                    =    1;       (* use R3D_SE_...       *)
  R3D_STA_DISABLES                   =    2;       (* use R3D_SE_...       *)
  R3D_STA_DRAW_BUFFER                =    3;       (* use R3D_DB_...       *)
  R3D_STA_Z_FUNCTION                 =    4;       (* use R3D_ZF_...       *)
  R3D_STA_SRC_BLEND_FUNCTION    = 5;      (* use R3D_BF_...       *)
  R3D_STA_DST_BLEND_FUNCTION    = 6;      (* use R3D_BF_...       *)
  R3D_STA_TEX_FILTER                 =    7;       (* use R3D_TX_...       *)
  R3D_STA_TEX_FUNCTION          = 8;      (* use R3D_TF_...       *)
  R3D_STA_ROP_FUNCTION          = 9;      (* use R3D_RF_...       *)


(*
// Arbitary valued states.
// Pick ONE of ...
*)
  R3D_STA_AREA_MASK           =           16;     (* Area pattern bitmask *)
  R3D_STA_LINE_MASK           =           48;     (* Line pattern bitmask *)
  R3D_STA_PIXEL_MASK          =           49;     (* RGBA or bitmask      *)
  R3D_STA_FG_COLOR            =           50;     (* long, RGBA or index  *)
  R3D_STA_BG_COLOR            =           51;     (* long, RGBA or index  *)
  R3D_STA_FG_DEPTH            =           52;     (* FIXED 16.16 for raster*)
  R3D_STA_BG_DEPTH            =           53;     (* FIXED 16.16 for zclear*)
  R3D_STA_TEX_LUMINANCE       =   54;     (* >> Will Go away!     *)
  R3D_STA_TEX_ID              =           55;     (* Handle for current texture.*)
  R3D_STA_SCISSOR_LEFT        =   56;
  R3D_STA_SCISSOR_TOP         =   57;
  R3D_STA_SCISSOR_RIGHT       =   58;     (* right,bottom, not inclusive*)
  R3D_STA_SCISSOR_BOTTOM      =   59;     (* Validity Check done here.    *)
  R3D_STA_ZOOM_X			       =60;
  R3D_STA_ZOOM_Y			       =61;

(*
// These are 3DR message type definitions
// Pick ONE of ...
*)
  R3D_MSG_NOP                 =                    0;
  R3D_MSG_CREATE_RC           =            1;
  R3D_MSG_DELETE_RC           =            2;
  R3D_MSG_GET_STATE           =            8;
  R3D_MSG_NEW_TEXTURE         =            9;
  R3D_MSG_LOAD_TEXTURE        =   10;
  R3D_MSG_FREE_TEXTURE        =   11;
  R3D_MSG_NEW_BITMAP          =           12;
  R3D_MSG_LOAD_BITMAP         =           13;
  R3D_MSG_FREE_BITMAP         =           14;
  R3D_MSG_PACKET              =           16;
  R3D_MSG_BITMAP              =           17;
  R3D_MSG_CACHED_BITMAP       =   18;

  R3D_MSG_SET_EXACT_PALETTE_ENTRY  =      32;
  R3D_MSG_SET_APPROX_PALETTE_ENTRY =      33;
  R3D_MSG_APPLY_NEW_PALETTE        =      34;

(*
// Possible values for: R3D_STA_ENABLES
//                      R3D_STA_DISABLES
// Logical OR of ...
*)
  R3D_SE_SHADING          =               $00000001;     (* interpolate r,g,b, a       *)
  R3D_SE_TEXTURING        =               $00000002;     (* interpolate u,v,w          *)
  R3D_SE_MODULATION       =               $00000004;     (* modulate color & texture   *)
  R3D_SE_Z_BUFFERING      =               $00000008;     (* interpolate z and compare  *)
  R3D_SE_ANTIALIASING     =               $00000010;     (* generate subpixel alpha    *)

  R3D_SE_PIXEL_MASKING    =       $00000020;     (* selective pixel update     *)
  R3D_SE_Z_MASKING        =               $00000040;     (* selective z update         *)
  R3D_SE_PATTERNING       =               $00000080;     (* selective pixel & z update *)
  R3D_SE_SCISSORING       =               $00000100;     (* selective pixel & z update *)
  R3D_SE_CLIPPING         =               $00000200;     (* selective pixel & z update *)

  R3D_SE_BLENDING         =               $00000400;     (* per-pixel operation        *)
  R3D_SE_RASTEROP         =               $00000800;     (* per-pixel operation        *)
  R3D_SE_DITHERING        =               $00001000;     (* per-pixel operation        *)

  R3D_SE_FILTERING        =               $00002000;     (* texture filter control     *)
  R3D_SE_TRANSPARENCY     =               $00004000;     (* texel use control          *)

  R3D_SE_SWAP_WITH_ZOOM		= $00080000;
  R3D_SE_COLOR_INDEXED_MODE	= $00100000;
  R3D_SE_NON_PERSPECTIVE_CORRECTION_MODE	= $00200000;
  R3D_SE_NON_SUBPIXEL_PRECISION_MODE		= $00400000;

(*
// Possible values for: R3D_STA_DRAW_BUFFER
// Pick one of ...
*)
  R3D_DB_FRONT            =                0;
  R3D_DB_BACK             =                        1;

  R3D_DB_FRONT_RIGHT      =                2;
  R3D_DB_BACK_RIGHT       =                3;

(*
// Possible values for: R3D_STA_Z_FUNCTION
// Pick one of ...
*)
  R3D_ZF_NEVER            =                0;
  R3D_ZF_LT               =                        1;
  R3D_ZF_EQ               =                        2;
  R3D_ZF_LE               =                        3;
  R3D_ZF_GT               =                        4;
  R3D_ZF_NE               =                        5;
  R3D_ZF_GE               =                        6;
  R3D_ZF_ALWAYS           =                7;

(*
// Possible values for: R3D_STA_SRC_BLEND_FUNCTION
//                      R3D_STA_DST_BLEND_FUNCTION
// Pick one of ...
*)
  R3D_BF_ZERO             =                        0;
  R3D_BF_ONE              =                        1;
  R3D_BF_SRC              =                        2;     (* Only R3D_STA_DST_BLEND_FUNCTION *)
  R3D_BF_SRC_INV          =                3;     (* Only R3D_STA_DST_BLEND_FUNCTION *)
  R3D_BF_SRC_ALPHA        =                4;
  R3D_BF_SRC_ALPHA_INV    =        5;
  R3D_BF_DST              =                        6;     (* Only R3D_STA_SRC_BLEND_FUNCTION *)
  R3D_BF_DST_INV          =                7;     (* Only R3D_STA_SRC_BLEND_FUNCTION *)
  R3D_BF_DST_ALPHA        =                8;
  R3D_BF_DST_ALPHA_INV    =        9;

(*
// Possible values for: R3D_STA_ROP_FUNCTION
// Pick one of ...
*)
  R3D_RF_CLEAR            =                0;
  R3D_RF_AND              =                        1;
  R3D_RF_AND_REVERSE      =                2;
  R3D_RF_COPY             =                        3;
  R3D_RF_AND_INVERT       =                4;
  R3D_RF_NOOP             =                        5;
  R3D_RF_XOR              =                        6;
  R3D_RF_OR               =                        7;
  R3D_RF_NOR              =                        8;
  R3D_RF_EQUIV            =                9;
  R3D_RF_INVERT           =               10;
  R3D_RF_OR_REVERSE       =               11;
  R3D_RF_COPY_INVERT      =               12;
  R3D_RF_OR_INVERT        =               13;
  R3D_RF_NAND             =                       14;
  R3D_RF_SET              =                       15;

(*
// Possible values for: R3D_STA_TEX_FUNCTION
// Pick ONE of ...
*)
  R3D_TF_REDUCE           =                1;
  R3D_TF_MULTIPLY         =                2;
  R3D_TF_SHADE            =                3;

(*
// Possible values for: R3D_STA_TEX_FILTER
// Pick ONE of ...
*)
  R3D_TX_BILINEAR         =                1;     (* interpolate 4 pixels         *)
  R3D_TX_CUSTOM           =                9;     (* your favorite filter         *)
  R3D_TX_MIPMAP_NEAREST   =       10;     (* nearest mipmap               *)
  R3D_TX_MIPMAP_LINEAR    =       11;     (* interpolate between mipmaps  *)
  R3D_TX_MIPMAP_BILINEAR  =       12;     (* interpolate 4x within mipmap *)
  R3D_TX_MIPMAP_TRILINEAR =       13;     (* interpolate mipmaps,4 pixels *)


(*
// Possible values for: RCInfo in the R3dCreateRC() prototype below
// Logical OR of ...
*)
  R3D_RI_DEFAULT          =               $0000;
  R3D_RI_DOUBLE_BUFFER    =       $0001;
  R3D_RI_COLOR_INDEXED    =       $0002;
  R3D_RI_EXTENDED         =               $0004;

  R3D_RI_RESERVED0        =               $0008;
  R3D_RI_RESERVED1        =               $0010;
  R3D_RI_RESERVED2        =               $0020;
  R3D_RI_RESERVED3        =               $0040;
  R3D_RI_RESERVED4        =               $0080;
  R3D_RI_RESERVED5        =               $0100;
  R3D_RI_RESERVED6        =               $0200;
  R3D_RI_RESERVED7        =               $0400;
  R3D_RI_RESERVED8        =               $0800;

  R3D_RI_STEREO           =               R3D_RI_RESERVED0;
  R3D_RI_ZBUFFER          =               R3D_RI_RESERVED8;

(*
// Possible values for : VtxInfo in the R3dPrimitive() prototype below
// Logical OR of ...
*)
  R3D_VI_FIXED            =               $0000;
  R3D_VI_FLOAT            =               $0001;
  R3D_VI_COLOR            =               $0002;
  R3D_VI_TEXTURE          =               $0004;

(*
// Possible values for: BmpInfo in the R3dBitmap() prototype below
// Logical OR of ...
*)
  R3D_BI_INDEX            =               $0001;
  R3D_BI_CHROMAKEY        =               $0002;
  R3D_BI_ALPHA            =               $0004;

(*
// Possible values for: SwapInfo in the R3dSwapBuffers() prototype below
// Logical OR of ...
*)
  R3D_SI_LEFT             =                       $0000;
  R3D_SI_RIGHT            =               $0001;

  R3D_SI_SCISSOR          =               $0004;

(*
// Possible values for: ClearInfo in the R3dClearBuffers() prototype below
// Logical OR of ...
*)
  R3D_CI_DRAW_BUFFER        =             $0001;

  R3D_CI_DRAW_BUFFER_LEFT   =     $0001;
  R3D_CI_DRAW_BUFFER_RIGHT  =     $0002;

  R3D_CI_ZBUFFER            =             $0004;

(*
// Possible values for: TexInfo in the R3dNewTexture() prototype below.
// Logical OR of ...
*)
  R3D_TI_DEFAULT            =             $0000;
  R3D_TI_MIPMAP             =             $0001;
  R3D_TI_HIRES              =             $0002;
  R3D_TI_KEEP               =                     $0004;
  R3D_TI_INDEX              =             $0010;
  R3D_TI_CHROMAKEY          =             $0020;
  R3D_TI_ALPHA              =             $0040;

(*
// Possible values for: R3dMsgHdr.MsgHdrVersion field
// Pick ONE of ...
*)
  R3D_MSG_VERSION           =             1;

(*
// Maximum No of vertices to be passed using R3dPrimitive()
*)
  R3D_MAX_VERTICES          =             256;

(*
//      3DR Function Prototypes :
*)

(*
function R3dBegin : longint;
procedure R3dEnd;
*)
function R3dCreateRC(hDC : TR3dHandle; RCInfo : word) : longint; stdcall;

function R3dDeleteRC(hRC : TR3dHandle) : longint; stdcall;

procedure R3dPrimitive(hRC : TR3dHandle;
  PrmType, VtxInfo, VtxCount, VtxSize : word;
  VtxPtr : pointer); stdcall;

function R3dGetState(hRC : TR3dHandle; State : word) : longint; stdcall;

procedure R3dSetState(hRC : TR3dHandle; State : word; Date : longint); stdcall;

function R3dNewTexture(hRC : TR3dHandle;
  TexInfo, TexBitsPixel, TexWidth, TexHeight : word) : longint; stdcall;

function R3dLoadTexture(hRC : TR3dHandle;
  TexInfo, TexBitsPixel, TexMipLevel, TexWidth, TexRows : word;
  TexWidthBytes : smallint; TexId : longint; TexData : longint;
  TexBuffer : pointer) : longint; stdcall;

procedure R3dFreeTexture(hRC : TR3dHandle; TexId : TR3dHandle); stdcall;

function R3dSetPaletteEntry(hRC : longint;
  PalEntry : longint; Exactness : longint) : longint; stdcall;

procedure R3dApplyNewPalette(hRC : longint); stdcall;

procedure R3dBitmap(hRC : TR3dHandle; BmpInfo, BmpBitsPixel, BmpWidth,
  BmpHeight : word; BmpWidthBytes, BmpSrcX, BmpSrcY, BmpDstX, BmpDstY : smallint;
  BmpData : longint; BmpBuffer : pointer); stdcall;

procedure R3dSwapBuffers(hRC : TR3dHandle; SwapInfo : word); stdcall;

procedure R3dClearBuffers(hRC : TR3dHandle; ClearInfo : word); stdcall;

function R3dMessage(hRC : TR3dHandle;
  InDataSize : longint; InDataPtr : pointer;
  OutDataSize : longint; OutDataPtr : pointer) : longint; stdcall;

procedure R3dCachedBitmap(hRC: TR3dHandle; BmpInfo: Word; BmpSrcX: SmallInt;
           BmpSrcY: SmallInt; BmpDstX: SmallInt; BmpDstY: SmallInt;
           BmpWidth: Word; BmpHeight: Word; BmpData: Longint;
           BmpId: Longint); stdcall;

function  R3dLoadBitmap(hRC: TR3dHandle; BmpInfo: Word; BmpBitsPixel: Word;
           BmpWidth: Word; BmpHeight: Word; BmpWidthBytes: SmallInt;
           BmpData: Longint; BmpBuffer: PByte): Longint; stdcall;


function  R3dNewBitmap(hRC: TR3dHandle; BmpInfo: Word; BmpBitsPixel: Word;
           BmpWidth: Word; BmpHeight: Word): Longint; stdcall;

procedure R3dFreeBitmap(hRC: TR3dHandle; BmpId: Longint); stdcall;

function DwordToFixedPt(D : longint) : TFixed32;
function FloatToFixedPt(F : single) : TFixed32;

function R3DColor(rr,gg,bb,aa : byte) : TR3DClr;

implementation

const
{$IFDEF WIN32}
  dll_3dr = '3DR32.DLL';
{$ELSE}
  dll_3dr = '3DR.DLL';
{$ENDIF}

{
function R3dBegin; external dll_3dr;
procedure R3dEnd; external dll_3dr;
}

function R3dCreateRC(hDC : TR3dHandle; RCInfo : word) : longint; external dll_3dr;

function R3dDeleteRC(hRC : TR3dHandle) : longint; external dll_3dr;

procedure R3dPrimitive(hRC : TR3dHandle;
  PrmType, VtxInfo, VtxCount, VtxSize : word;
  VtxPtr : pointer); external dll_3dr;

function R3dGetState(hRC : TR3dHandle; State : word) : longint; external dll_3dr;

procedure R3dSetState(hRC : TR3dHandle; State : word; Date : longint); external dll_3dr;

function R3dNewTexture(hRC : TR3dHandle;
  TexInfo, TexBitsPixel, TexWidth, TexHeight : word) : longint; external dll_3dr;

function R3dLoadTexture(hRC : TR3dHandle;
  TexInfo, TexBitsPixel, TexMipLevel, TexWidth, TexRows : word;
  TexWidthBytes : smallint; TexId : longint; TexData : longint;
  TexBuffer : pointer) : longint; external dll_3dr;

procedure R3dFreeTexture(hRC : TR3dHandle; TexId : TR3dHandle); external dll_3dr;

function R3dSetPaletteEntry(hRC : longint;
  PalEntry : longint; Exactness : longint) : longint; external dll_3dr;

procedure R3dApplyNewPalette(hRC : longint); external dll_3dr;

procedure R3dBitmap(hRC : TR3dHandle; BmpInfo, BmpBitsPixel, BmpWidth,
  BmpHeight : word; BmpWidthBytes, BmpSrcX, BmpSrcY, BmpDstX, BmpDstY : smallint;
  BmpData : longint; BmpBuffer : pointer); external dll_3dr;

procedure R3dSwapBuffers(hRC : TR3dHandle; SwapInfo : word); external dll_3dr;

procedure R3dClearBuffers(hRC : TR3dHandle; ClearInfo : word); external dll_3dr;

function R3dMessage(hRC : TR3dHandle;
  InDataSize : longint; InDataPtr : pointer;
  OutDataSize : longint; OutDataPtr : pointer) : longint; external dll_3dr;

procedure R3dFreeBitmap; external dll_3dr;
function R3dNewBitmap; external dll_3dr;
function R3dLoadBitmap; external dll_3dr;
procedure R3dCachedBitmap; external dll_3dr;

function DwordToFixedPt(D : longint) : TFixed32;
begin
  Result := TFixed32((D and $FFFF) shl 16);
end;

function FloatToFixedPt(F : single) : TFixed32;
begin
  Result := TFixed32(round(F*65536.0));
end;

function R3DColor(rr,gg,bb,aa : byte) : TR3DClr;
begin
  with Result do begin
    r := rr+word(rr) shl 8;
    g := gg+word(gg) shl 8;
    b := bb+word(bb) shl 8;
    a := aa+word(aa) shl 8;
  end;
end;

end.
